import unittest

from . import FixerTestCase


class testFixDivisionTrace(FixerTestCase):
  fixer = "division_trace"

  def test_simple_division(self):
    b = """10 / 20"""
    a = """division_traced("<string>",1,0,10 , 20)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int')])

  def test_nested_divisions(self):
    b = """10 / 20 / 30"""
    a = """division_traced("<string>",1,1,division_traced("<string>",1,0,10 , 20) , 30)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0), (u'<string>', 1, 1)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int'), (u'<string>', 1, 1, u'int', u'int')])

  def test_nested_divisions_with_parentheses_1(self):
    b = """(10 / 20) / 30"""
    a = """division_traced("<string>",1,1,(division_traced("<string>",1,0,10 , 20)) , 30)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0), (u'<string>', 1, 1)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int'), (u'<string>', 1, 1, u'int', u'int')])

  def test_nested_divisions_with_parentheses_2(self):
    b = """30 / (20 / 10)"""
    a = """division_traced("<string>",1,1,30 , (division_traced("<string>",1,0,20 , 10)))"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0), (u'<string>', 1, 1)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int'), (u'<string>', 1, 1, u'int', u'int')])

  def test_inline_division_1(self):
    b = """10 / 20 / 30"""
    a = """division_traced("<string>",1,1,division_traced("<string>",1,0,10 , 20) , 30)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0), (u'<string>', 1, 1)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int'), (u'<string>', 1, 1, u'int', u'int')])

  def test_inline_division_2(self):
    b = """10 / 20 * 30"""
    a = """division_traced("<string>",1,0,10 , 20) * 30"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int')])

  def test_inline_division_3(self):
    b = """10 * 20 / 30"""
    a = """division_traced("<string>",1,0,10 * 20 , 30)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int')])

  def test_division_on_line_continuation(self):
    b = """10 \
  / 20"""
    a = """division_traced("<string>",1,0,10 \
  , 20)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int')])

  def test_multiline_division(self):
    b = """foo = \
  10 / 20"""
    a = """foo = \
  division_traced("<string>",1,0,10 , 20)"""
    self.check(b, a)

    self.exec_code(a)
    self.assertDataEqual("division_modified", [(u'<string>', 1, 0)])
    self.assertDataEqual("division_trace", [(u'<string>', 1, 0, u'int', u'int')])


if __name__ == '__main__':
  unittest.main()
