import unittest

from . import FixerTestCase


class testFixNestedExceptTrace(FixerTestCase):
  fixer = "nested_except_trace"

  def test_try(self):
    a = """
        try:
          try:
            1
          except Exception as e:
            2
        except Exception as e:
          3
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [])

  def test_except(self):
    a = """
        try:
          1
        except Exception as e:
          try:
            2
          except Exception as e:
            3
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [(u'<string>', None, None, 4, 7)])

  def test_except_2(self):
    a = """
        try:
          1
        except Exception as e:
          try:
            2
          except Exception as f:
            3
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [])

  def test_multiple_except(self):
    a = """
        try:
          1
        except Exception as e:
          try:
            2
          except Exception as e:
            try:
              3
            except Exception as e:
              4
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [(u'<string>', None, None, 7, 10), (u'<string>', None, None, 4, 7), (u'<string>', None, None, 4, 10)])

  def test_else(self):
    a = """
        try:
          1
        except Exception as e:
          2
        else:
          try:
            3
          except Exception as e:
            4
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [])

  def test_finally(self):
    a = """
        try:
          1
        except Exception as e:
          2
        finally:
          try:
            3
          except Exception as e:
            4
        """
    self.unchanged(a)
    self.assertDataEqual("nested_except_trace", [])


if __name__ == '__main__':
  unittest.main()
